IMPLEMENTATION MODULE V24; (* 19.05.85 / Franz Kronseder *)
                       (* last modification 24.07.85 *)
                       (* Asynchronous Interface to Macintosh modem port *)
                       (* full duplex, using .Ain and .AOut channels     *)
                       (* identical to V24.DEF on SMAKY8 *)

FROM SYSTEM IMPORT ADDRESS,ADR;
IMPORT Terminal;
FROM OutTerminal IMPORT WriteI;
FROM OStrings IMPORT msg,mkOStr,OStr63;
FROM pbfile IMPORT PBOpen,PBClose,PBControl,PBStatus,PBRead,PBWrite,
                   PBKillIO;
FROM pbfile IMPORT  ParamBlockRec,ASStaRec;
FROM DeviceMgr IMPORT DriverOpen,DriverClose,DriverRead,DriverWrite,
                      DriverControl,DriverStatus,DriverKillIO,
                      OpParamPtr,OpParamType;

FROM MacBase IMPORT MacNIL;

CONST NoErr = 0; (* File System Error Code *)
      cr=0DH; lf=0AH;    (* ascii carriage return, linefeed *)
TYPE StrPtr = ADDRESS; LongInt=ADDRESS; Ptr=ADDRESS; OsErr=INTEGER;

VAR ch:CHAR;
    DriverName:OStr63;
    AInrefNum,AOutrefNum:INTEGER;
    SerialInputBuffer : ARRAY[1..1000] OF CHAR;
TYPE PBProc = PROCEDURE (VAR ParamBlockRec,BOOLEAN): OsErr;
PROCEDURE PBCall (PBName:PBProc; VAR paramBlock: ParamBlockRec): OsErr;
 (* perform synchronuous low level File-System Call *)
 BEGIN RETURN PBName(paramBlock,FALSE); END PBCall;

 PROCEDURE errtest(VAR msg:ARRAY OF CHAR; error:INTEGER);
  VAR ch:CHAR;
  BEGIN IF error<>0 THEN Terminal.WriteString("--- ERROR ");
                         Terminal.WriteString(msg);
                         Terminal.Write(" "); WriteI(error,4);
                         Terminal.Read(ch); Terminal.WriteLn;
                    END;(*if*)
  END errtest;
(*-------------------------------------------------------------------------*)


  PROCEDURE installInputBuffer;
   VAR ParamRec:OpParamType; echo:OsErr;
   BEGIN WITH ParamRec DO
     asncBPtr:=ADR(SerialInputBuffer); asncBLen:=1000;
     echo:=DriverControl(AInrefNum,9,ADR(ParamRec));
     errtest("installBuffer",echo);
     (**Terminal.WriteString("install InputBuffer size=1000 Bytes");  Terminal.WriteLn;**)
   END END installInputBuffer;

  PROCEDURE SetFlow;
   CONST FlowControlCode = 10;
   VAR  count:ADDRESS; AInControl,AOutControl:OpParamType;echo:OsErr;
   BEGIN
    WITH AInControl.flgs DO
     fXOn :=CHR(000H);
     fCTS :=CHR(000H);
     xon  :=CHR(11H);
     xoff :=CHR(13H);
     errs :=CHR(000H);
     fInX :=CHR(0FFH);
    END (*with*);
    echo:=DriverControl(AInrefNum,FlowControlCode,ADR(AInControl));
    errtest("SetFlow",echo);
    WITH AOutControl.flgs DO
     fXOn :=CHR(0FFH);
     fCTS :=CHR(000H);
     xon  :=CHR(11H);
     xoff :=CHR(13H);
     errs :=CHR(000H);
     fInX :=CHR(000H);
    END (*with*);
    echo:=DriverControl(AOutrefNum,FlowControlCode,ADR(AOutControl));
    errtest("SetFlow",echo);
   END SetFlow;

  PROCEDURE ConfigWord(baudrate:CARDINAL):CARDINAL;
   (* calculates an appropriate configureation word for the serial driver *)
   (* according to the given baud rate *)
   CONST default = 0CC0AH; (* 9600 baud, 8 data, 2 stop, no parity *)
   VAR   baud:CARDINAL;
   BEGIN    IF baudrate=0      THEN baud :=00AH
         ELSIF baudrate=57600  THEN baud :=000H
         ELSIF baudrate=19200  THEN baud :=004H
         ELSIF baudrate= 9600  THEN baud :=00AH
         ELSIF baudrate= 7200  THEN baud :=00EH
         ELSIF baudrate= 4800  THEN baud :=016H
         ELSIF baudrate= 3600  THEN baud :=01EH
         ELSIF baudrate= 2400  THEN baud :=02EH
         ELSIF baudrate= 1800  THEN baud :=03EH
         ELSIF baudrate= 1200  THEN baud :=05EH
         ELSIF baudrate=  600  THEN baud :=0BDH
         ELSIF baudrate=  300  THEN baud :=17CH
                               ELSE baud :=00AH END;(*if*)
         RETURN 0CC00H+baud
    END ConfigWord;

  PROCEDURE SetBaudRate(refNum:INTEGER; baud:CARDINAL):OsErr;
   CONST configCode=8;
   VAR echo:OsErr; ParamRec:OpParamType;
   BEGIN  ParamRec.asncConfig:=INTEGER(ConfigWord(baud));
          echo:=DriverControl(refNum,configCode,ADR(ParamRec));
          RETURN echo;
   END SetBaudRate;
(*----------------------------------------------------------------------------*)
(** here come the exported procedures **)

PROCEDURE Initialize (baud:CARDINAL);
 (* baud may be 0 for default, or 300,600,1200,2400,4800,9600,19200,57600 *)

  VAR DriverName:OStr63;  echo:OsErr;
  BEGIN (* Terminal.WriteLn;
           Terminal.WriteString("V24.Initialize /07.07.85fxk"); Terminal.WriteLn;*)
      mkOStr(".AIn",DriverName);
      echo:=DriverOpen(ADR(DriverName),AInrefNum);
      (* Terminal.WriteString(".AIn  opened. refNum =");
         WriteI(AInrefNum,4); Terminal.WriteLn;*)
      mkOStr(".AOut",DriverName);
      echo:=DriverOpen(ADR(DriverName),AOutrefNum);
      (* Terminal.WriteString(".AOut opened. refNum =");
         WriteI(AOutrefNum,4); Terminal.WriteLn; *)
      echo:=SetBaudRate(AInrefNum,baud); echo:=SetBaudRate(AOutrefNum,baud);

      installInputBuffer;
  END Initialize;

PROCEDURE Break;
 CONST setBREAK=12; clearBREAK=11;
 VAR echo:OsErr; ch:CHAR; count:CARDINAL;
 (* send a BREAK Signal on the output channel *)
BEGIN echo:=DriverControl(AOutrefNum,setBREAK,MacNIL);
      errtest("setBreak",echo);
      FOR count:=1 TO 65000 DO  END; (* idle some time *)
      echo:=DriverControl(AOutrefNum,clearBREAK,MacNIL);
      errtest("clearBreak",echo);
END Break;

PROCEDURE Write(ch:CHAR);
  VAR count:LongInt;echo:OsErr;
BEGIN
   count:=1; echo:=DriverWrite(AOutrefNum,count,ADR(ch));
END Write;

PROCEDURE Read(VAR ch:CHAR);
  VAR count:LongInt;echo:OsErr;
 BEGIN
   count:=1; echo:=DriverRead(AInrefNum,count,ADR(ch));
 END Read;

PROCEDURE BusyRead(VAR ch:CHAR; VAR got:BOOLEAN);
 CONST BufferCountCode = 2;
 VAR  count:ADDRESS; AInStatus:OpParamType;        echo:OsErr;
 BEGIN WITH AInStatus DO
        echo:=DriverStatus(AInrefNum,BufferCountCode,ADR(AInStatus));
        IF  asyncNBytes>0
         THEN count:=1; echo:=DriverRead(AInrefNum,count,ADR(ch));
              got:=TRUE;
         ELSE got:=FALSE
         END; (*if*)
 END; END BusyRead;


END V24.
